@@ -21,9 +21,11 @@ import argparse
 import atexit
 from dataclasses import dataclass
 import enum
+import json
 import logging
 import os
 from pathlib import Path
+import platform
 import re
 import shlex
 import subprocess
@@ -92,14 +94,9 @@ def _parse_args() -> argparse.Namespace:
         'json files.',
     )
     parser.add_argument(
-        '--python-interpreter',
+        '--python-virtualenv-config',
         type=Path,
-        help='Python interpreter to use for this action.',
-    )
-    parser.add_argument(
-        '--python-virtualenv',
-        type=Path,
-        help='Path to a virtualenv to use for this action.',
+        help='Path to a virtualenv json config to use for this action.',
     )
     parser.add_argument(
         'original_cmd',
@@ -114,6 +111,11 @@ def _parse_args() -> argparse.Namespace:
     return parser.parse_args()
 
 
+def _abspath(path: Path) -> Path:
+    """Turns a path into an absolute path, not resolving symlinks."""
+    return Path(os.path.abspath(path))
+
+
 class GnPaths(NamedTuple):
     """The set of paths needed to resolve GN paths to filesystem paths."""
     root: Path
@@ -126,9 +128,9 @@ class GnPaths(NamedTuple):
     def resolve(self, gn_path: str) -> Path:
         """Resolves a GN path to a filesystem path."""
         if gn_path.startswith('//'):
-            return self.root.joinpath(gn_path.lstrip('/')).resolve()
+            return _abspath(self.root.joinpath(gn_path.lstrip('/')))
 
-        return self.cwd.joinpath(gn_path).resolve()
+        return _abspath(self.cwd.joinpath(gn_path))
 
     def resolve_paths(self, gn_paths: str, sep: str = ';') -> str:
         """Resolves GN paths to filesystem paths in a delimited string."""
@@ -169,7 +171,7 @@ class Label:
 
         # Resolve the directory to an absolute path
         set_attr('dir', paths.resolve(directory))
-        set_attr('relative_dir', self.dir.relative_to(paths.root.resolve()))
+        set_attr('relative_dir', self.dir.relative_to(_abspath(paths.root)))
 
         set_attr(
             'out_dir',
@@ -429,7 +431,7 @@ def _target_objects(paths: GnPaths, expr: _Expression) -> _Actions:
         yield _ArgAction.EMIT_NEW, str(obj)
 
 
-# TODO(pwbug/347): Replace expressions with native GN features when possible.
+# TODO(b/234886742): Replace expressions with native GN features when possible.
 _FUNCTIONS: Dict['str', Callable[[GnPaths, _Expression], _Actions]] = {
     'TARGET_FILE': _target_file,
     'TARGET_FILE_IF_EXISTS': _target_file_if_exists,
@@ -533,6 +535,12 @@ class MissingPythonDependency(Exception):
     """An error occurred while processing a Python dependency."""
 
 
+def _load_virtualenv_config(json_file_path: Path) -> Tuple[str, str]:
+    with json_file_path.open() as json_fp:
+        json_dict = json.load(json_fp)
+    return json_dict.get('interpreter'), json_dict.get('path')
+
+
 def main(  # pylint: disable=too-many-arguments,too-many-branches,too-many-locals
     gn_root: Path,
     current_path: Path,
@@ -542,8 +550,7 @@ def main(  # pylint: disable=too-many-arguments,too-many-branches,too-many-local
     module: Optional[str],
     env: Optional[List[str]],
     python_dep_list_files: List[Path],
-    python_interpreter: Optional[Path],
-    python_virtualenv: Optional[Path],
+    python_virtualenv_config: Optional[Path],
     capture_output: bool,
     touch: Optional[Path],
     working_directory: Optional[Path],
@@ -565,7 +572,12 @@ def main(  # pylint: disable=too-many-arguments,too-many-branches,too-many-local
                 raise MissingPythonDependency(
                     'Unable to find top level source dir for the Python '
                     f'package "{pkg}"')
-            python_paths_list.append(top_level_source_dir.parent.resolve())
+            # Don't add this dir to the PYTHONPATH if no __init__.py exists.
+            init_py_files = top_level_source_dir.parent.glob('*/__init__.py')
+            if not any(init_py_files):
+                continue
+            python_paths_list.append(_abspath(top_level_source_dir.parent))
+
         # Sort the PYTHONPATH list, it will be in a different order each build.
         python_paths_list = sorted(python_paths_list)
 
@@ -574,15 +586,22 @@ def main(  # pylint: disable=too-many-arguments,too-many-branches,too-many-local
         return 1
 
     # GN build scripts are executed from the root build directory.
-    root_build_dir = Path.cwd().resolve()
+    root_build_dir = _abspath(Path.cwd())
 
     tool = current_toolchain if current_toolchain != default_toolchain else ''
-    paths = GnPaths(root=gn_root.resolve(),
+    paths = GnPaths(root=_abspath(gn_root),
                     build=root_build_dir,
-                    cwd=current_path.resolve(),
+                    cwd=_abspath(current_path),
                     toolchain=tool)
 
     command = [sys.executable]
+
+    python_interpreter = None
+    python_virtualenv = None
+    if python_virtualenv_config:
+        python_interpreter, python_virtualenv = _load_virtualenv_config(
+            python_virtualenv_config)
+
     if python_interpreter is not None:
         command = [str(root_build_dir / python_interpreter)]
 
@@ -607,10 +626,18 @@ def main(  # pylint: disable=too-many-arguments,too-many-branches,too-many-local
     is_pip_command = (module == 'pip'
                       or 'pip_install_python_deps.py' in script_command)
 
-    if python_paths_list and not is_pip_command:
-        existing_env = (run_args['env']
-                        if 'env' in run_args else os.environ.copy())
+    existing_env = (run_args['env']
+                    if 'env' in run_args else os.environ.copy())
+    new_env = {}
+    if python_virtualenv:
+        new_env['VIRTUAL_ENV'] = str(root_build_dir / python_virtualenv)
+        bin_folder = 'Scripts' if platform.system() == 'Windows' else 'bin'
+        new_env['PATH'] = os.pathsep.join([
+            str(root_build_dir / python_virtualenv / bin_folder),
+            existing_env.get('PATH', '')
+        ])
 
+    if python_virtualenv and python_paths_list and not is_pip_command:
         python_path_prepend = os.pathsep.join(
             str(p) for p in set(python_paths_list))
 
@@ -619,26 +646,12 @@ def main(  # pylint: disable=too-many-arguments,too-many-branches,too-many-local
             path_str for path_str in
             [python_path_prepend,
              existing_env.get('PYTHONPATH', '')] if path_str)
-        new_env = {
-            'PYTHONPATH': new_python_path,
-            # mypy doesn't use PYTHONPATH for analyzing imports so module
-            # directories must be added to the MYPYPATH environment variable.
-            'MYPYPATH': new_python_path,
-        }
-        # print('PYTHONPATH')
-        # for ppath in python_paths_list:
-        #     print(str(ppath))
 
-        if python_virtualenv:
-            new_env['VIRTUAL_ENV'] = str(root_build_dir / python_virtualenv)
-            new_env['PATH'] = os.pathsep.join([
-                str(root_build_dir / python_virtualenv / 'bin'),
-                existing_env.get('PATH', '')
-            ])
+        new_env['PYTHONPATH'] = new_python_path
 
-        if 'env' not in run_args:
-            run_args['env'] = {}
-        run_args['env'].update(new_env)
+    if 'env' not in run_args:
+        run_args['env'] = {}
+    run_args['env'].update(new_env)
 
     if capture_output:
         # Combine stdout and stderr so that error messages are correctly
@@ -657,8 +670,8 @@ def main(  # pylint: disable=too-many-arguments,too-many-branches,too-many-local
     if working_directory:
         run_args['cwd'] = working_directory
 
-    # TODO(pwbug/666): Deprecate the --lockfile option as part of the Python GN
-    # template refactor.
+    # TODO(b/235239674): Deprecate the --lockfile option as part of the Python
+    # GN template refactor.
     if lockfile:
         try:
             acquire_lock(lockfile, is_pip_command)
