@@ -143,6 +143,9 @@ class LogView:
         # Cache of formatted text tuples used in the last UI render.
         self._line_fragment_cache: List[StyleAndTextTuples] = []
 
+        # Check if any logs are already in the log_store and update the view.
+        self.new_logs_arrived()
+
     def view_mode_changed(self) -> None:
         self._reset_log_screen_on_next_render = True
 
@@ -735,7 +738,7 @@ class LogView:
         """Get pre-formatted table header."""
         return self.log_store.render_table_header()
 
-    def render_content(self) -> list:
+    def render_content(self) -> List:
         """Return logs to display on screen as a list of FormattedText tuples.
 
         This function determines when the log screen requires re-rendeing based
