@@ -21,9 +21,10 @@ import logging
 import os
 from pathlib import Path
 import re
+import shutil
 import subprocess
 import sys
-from typing import Sequence, IO, Tuple, Optional, Callable, List
+from typing import Callable, Iterable, List, Sequence, TextIO
 
 try:
     import pw_presubmit
@@ -45,21 +46,25 @@ from pw_presubmit import (
     call,
     filter_paths,
     inclusive_language,
+    npm_presubmit,
     plural,
+    presubmit,
     PresubmitContext,
     PresubmitFailure,
     Programs,
     python_checks,
+    shell_checks,
+    todo_check,
 )
-from pw_presubmit.install_hook import install_hook
+from pw_presubmit.install_hook import install_git_hook
 
 _LOG = logging.getLogger(__name__)
 
 pw_package.pigweed_packages.initialize()
 
 # Trigger builds if files with these extensions change.
-_BUILD_EXTENSIONS = ('.py', '.rst', '.gn', '.gni',
-                     *format_code.C_FORMAT.extensions)
+_BUILD_FILE_FILTER = presubmit.FileFilter(
+    suffix=(*format_code.C_FORMAT.extensions, '.py', '.rst', '.gn', '.gni'))
 
 
 def _at_all_optimization_levels(target):
@@ -71,116 +76,163 @@ def _at_all_optimization_levels(target):
 # Build presubmit checks
 #
 def gn_clang_build(ctx: PresubmitContext):
-    build.gn_gen(ctx.root,
-                 ctx.output_dir,
-                 pw_RUN_INTEGRATION_TESTS=(sys.platform != 'win32'))
-    build.ninja(ctx.output_dir, *_at_all_optimization_levels('host_clang'))
+    build_targets = list(_at_all_optimization_levels('host_clang'))
+
+    # TODO(b/240982565): SocketStream currently requires Linux.
+    if sys.platform.startswith('linux'):
+        build_targets.append('integration_tests')
+
+    build.gn_gen(ctx)
+    build.ninja(ctx, *build_targets)
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_gcc_build(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, *_at_all_optimization_levels('host_gcc'))
+    build.gn_gen(ctx)
+    build.ninja(ctx, *_at_all_optimization_levels('host_gcc'))
 
 
 _HOST_COMPILER = 'gcc' if sys.platform == 'win32' else 'clang'
 
 
 def gn_host_build(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir,
-                *_at_all_optimization_levels(f'host_{_HOST_COMPILER}'))
+    build.gn_gen(ctx)
+    build.ninja(ctx, *_at_all_optimization_levels(f'host_{_HOST_COMPILER}'))
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_quick_build_check(ctx: PresubmitContext):
     """Checks the state of the GN build by running gn gen and gn check."""
-    build.gn_gen(ctx.root, ctx.output_dir)
+    build.gn_gen(ctx)
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
-def gn_full_build_check(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, *_at_all_optimization_levels('stm32f429i'),
-                *_at_all_optimization_levels(f'host_{_HOST_COMPILER}'),
-                'python.tests', 'python.lint', 'docs', 'fuzzers',
-                'pw_env_setup:build_pigweed_python_source_tree')
+@_BUILD_FILE_FILTER.apply_to_check()
+def gn_full_build_check(ctx: PresubmitContext) -> None:
+    build_targets = [
+        *_at_all_optimization_levels('stm32f429i'),
+        *_at_all_optimization_levels(f'host_{_HOST_COMPILER}'),
+        'python.tests',
+        'python.lint',
+        'docs',
+        'fuzzers',
+        'pw_env_setup:pypi_pigweed_python_source_tree',
+    ]
+
+    # TODO(b/234645359): Re-enable on Windows when compatibility tests build.
+    if sys.platform != 'win32':
+        build_targets.append('cpp14_compatibility')
+        build_targets.append('cpp20_compatibility')
+
+    # TODO(b/240982565): SocketStream currently requires Linux.
+    if sys.platform.startswith('linux'):
+        build_targets.append('integration_tests')
+
+    build.gn_gen(ctx)
+    build.ninja(ctx, *build_targets)
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_full_qemu_check(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir)
+    build.gn_gen(ctx)
     build.ninja(
-        ctx.output_dir,
+        ctx,
         *_at_all_optimization_levels('qemu_gcc'),
-        # TODO(pwbug/321) Re-enable clang.
-        # *_at_all_optimization_levels('qemu_clang'),
+        *_at_all_optimization_levels('qemu_clang'),
     )
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
+def gn_combined_build_check(ctx: PresubmitContext) -> None:
+    build_targets = [
+        *_at_all_optimization_levels('stm32f429i'),
+        *_at_all_optimization_levels(f'host_{_HOST_COMPILER}'),
+        'python.tests',
+        'python.lint',
+        'docs',
+        'fuzzers',
+        'pw_env_setup:pypi_pigweed_python_source_tree',
+    ]
+
+    # TODO(b/234645359): Re-enable on Windows when compatibility tests build.
+    if sys.platform != 'win32':
+        build_targets.append('cpp14_compatibility')
+        build_targets.append('cpp20_compatibility')
+
+    if sys.platform != 'win32':
+        build_targets.extend(_at_all_optimization_levels('qemu_gcc'))
+        build_targets.extend(_at_all_optimization_levels('qemu_clang'))
+
+    # TODO(b/240982565): SocketStream currently requires Linux.
+    if sys.platform.startswith('linux'):
+        build_targets.append('integration_tests')
+
+    build.gn_gen(ctx)
+    build.ninja(ctx, *build_targets)
+
+
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_arm_build(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, *_at_all_optimization_levels('stm32f429i'))
+    build.gn_gen(ctx)
+    build.ninja(ctx, *_at_all_optimization_levels('stm32f429i'))
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def stm32f429i(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir, pw_use_test_server=True)
+    build.gn_gen(ctx, pw_use_test_server=True)
     with build.test_server('stm32f429i_disc1_test_server', ctx.output_dir):
-        build.ninja(ctx.output_dir, *_at_all_optimization_levels('stm32f429i'))
+        build.ninja(ctx, *_at_all_optimization_levels('stm32f429i'))
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_boringssl_build(ctx: PresubmitContext):
     build.install_package(ctx.package_root, 'boringssl')
-    build.gn_gen(ctx.root,
-                 ctx.output_dir,
+    build.gn_gen(ctx,
                  dir_pw_third_party_boringssl='"{}"'.format(ctx.package_root /
                                                             'boringssl'))
     build.ninja(
-        ctx.output_dir,
+        ctx,
         *_at_all_optimization_levels('stm32f429i'),
         *_at_all_optimization_levels('host_clang'),
     )
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_nanopb_build(ctx: PresubmitContext):
     build.install_package(ctx.package_root, 'nanopb')
-    build.gn_gen(ctx.root,
-                 ctx.output_dir,
+    build.gn_gen(ctx,
                  dir_pw_third_party_nanopb='"{}"'.format(ctx.package_root /
                                                          'nanopb'))
     build.ninja(
-        ctx.output_dir,
+        ctx,
         *_at_all_optimization_levels('stm32f429i'),
         *_at_all_optimization_levels('host_clang'),
     )
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_crypto_mbedtls_build(ctx: PresubmitContext):
     build.install_package(ctx.package_root, 'mbedtls')
     build.gn_gen(
-        ctx.root,
-        ctx.output_dir,
-        pw_RUN_INTEGRATION_TESTS=False,
+        ctx,
         dir_pw_third_party_mbedtls='"{}"'.format(ctx.package_root / 'mbedtls'),
         pw_crypto_SHA256_BACKEND='"{}"'.format(ctx.root /
                                                'pw_crypto:sha256_mbedtls'),
         pw_crypto_ECDSA_BACKEND='"{}"'.format(ctx.root /
                                               'pw_crypto:ecdsa_mbedtls'))
-    build.ninja(ctx.output_dir)
+    build_targets = [*_at_all_optimization_levels(f'host_{_HOST_COMPILER}')]
+
+    # TODO(b/240982565): SocketStream currently requires Linux.
+    if sys.platform.startswith('linux'):
+        build_targets.append('integration_tests')
+
+    build.ninja(ctx, *build_targets)
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_crypto_boringssl_build(ctx: PresubmitContext):
     build.install_package(ctx.package_root, 'boringssl')
     build.gn_gen(
-        ctx.root,
-        ctx.output_dir,
-        pw_RUN_INTEGRATION_TESTS=False,
+        ctx,
         dir_pw_third_party_boringssl='"{}"'.format(ctx.package_root /
                                                    'boringssl'),
         pw_crypto_SHA256_BACKEND='"{}"'.format(ctx.root /
@@ -188,46 +240,54 @@ def gn_crypto_boringssl_build(ctx: PresubmitContext):
         pw_crypto_ECDSA_BACKEND='"{}"'.format(ctx.root /
                                               'pw_crypto:ecdsa_boringssl'),
     )
-    build.ninja(ctx.output_dir)
+    build_targets = [*_at_all_optimization_levels(f'host_{_HOST_COMPILER}')]
+
+    # TODO(b/240982565): SocketStream currently requires Linux.
+    if sys.platform.startswith('linux'):
+        build_targets.append('integration_tests')
+
+    build.ninja(ctx, *build_targets)
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_crypto_micro_ecc_build(ctx: PresubmitContext):
     build.install_package(ctx.package_root, 'micro-ecc')
     build.gn_gen(
-        ctx.root,
-        ctx.output_dir,
-        pw_RUN_INTEGRATION_TESTS=False,
+        ctx,
         dir_pw_third_party_micro_ecc='"{}"'.format(ctx.package_root /
                                                    'micro-ecc'),
         pw_crypto_ECDSA_BACKEND='"{}"'.format(ctx.root /
                                               'pw_crypto:ecdsa_uecc'),
     )
-    build.ninja(ctx.output_dir)
+    build_targets = [*_at_all_optimization_levels(f'host_{_HOST_COMPILER}')]
+
+    # TODO(b/240982565): SocketStream currently requires Linux.
+    if sys.platform.startswith('linux'):
+        build_targets.append('integration_tests')
+
+    build.ninja(ctx, *build_targets)
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_teensy_build(ctx: PresubmitContext):
     build.install_package(ctx.package_root, 'teensy')
-    build.gn_gen(ctx.root,
-                 ctx.output_dir,
+    build.gn_gen(ctx,
                  pw_arduino_build_CORE_PATH='"{}"'.format(str(
                      ctx.package_root)),
                  pw_arduino_build_CORE_NAME='teensy',
                  pw_arduino_build_PACKAGE_NAME='teensy/avr',
                  pw_arduino_build_BOARD='teensy40')
-    build.ninja(ctx.output_dir, *_at_all_optimization_levels('arduino'))
+    build.ninja(ctx, *_at_all_optimization_levels('arduino'))
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_software_update_build(ctx: PresubmitContext):
     build.install_package(ctx.package_root, 'nanopb')
     build.install_package(ctx.package_root, 'protobuf')
     build.install_package(ctx.package_root, 'mbedtls')
     build.install_package(ctx.package_root, 'micro-ecc')
     build.gn_gen(
-        ctx.root,
-        ctx.output_dir,
+        ctx,
         dir_pw_third_party_protobuf='"{}"'.format(ctx.package_root /
                                                   'protobuf'),
         dir_pw_third_party_nanopb='"{}"'.format(ctx.package_root / 'nanopb'),
@@ -239,54 +299,53 @@ def gn_software_update_build(ctx: PresubmitContext):
         pw_crypto_SHA256_BACKEND='"{}"'.format(ctx.root /
                                                'pw_crypto:sha256_mbedtls'))
     build.ninja(
-        ctx.output_dir,
+        ctx,
         *_at_all_optimization_levels('host_clang'),
     )
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_pw_system_demo_build(ctx: PresubmitContext):
     build.install_package(ctx.package_root, 'freertos')
     build.install_package(ctx.package_root, 'nanopb')
     build.install_package(ctx.package_root, 'stm32cube_f4')
     build.gn_gen(
-        ctx.root,
-        ctx.output_dir,
+        ctx,
         dir_pw_third_party_freertos='"{}"'.format(ctx.package_root /
                                                   'freertos'),
         dir_pw_third_party_nanopb='"{}"'.format(ctx.package_root / 'nanopb'),
         dir_pw_third_party_stm32cube_f4='"{}"'.format(ctx.package_root /
                                                       'stm32cube_f4'),
     )
-    build.ninja(ctx.output_dir, 'pw_system_demo')
+    build.ninja(ctx, 'pw_system_demo')
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_qemu_build(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, *_at_all_optimization_levels('qemu_gcc'))
+    build.gn_gen(ctx)
+    build.ninja(ctx, *_at_all_optimization_levels('qemu_gcc'))
 
 
-@filter_paths(endswith=_BUILD_EXTENSIONS)
+@_BUILD_FILE_FILTER.apply_to_check()
 def gn_qemu_clang_build(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, *_at_all_optimization_levels('qemu_clang'))
+    build.gn_gen(ctx)
+    build.ninja(ctx, *_at_all_optimization_levels('qemu_clang'))
 
 
 def gn_docs_build(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, 'docs')
+    build.gn_gen(ctx)
+    build.ninja(ctx, 'docs')
 
 
 def gn_host_tools(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, 'host_tools')
+    build.gn_gen(ctx)
+    build.ninja(ctx, 'host_tools')
 
 
 @filter_paths(endswith=format_code.C_FORMAT.extensions)
 def oss_fuzz_build(ctx: PresubmitContext):
-    build.gn_gen(ctx.root, ctx.output_dir, pw_toolchain_OSS_FUZZ_ENABLED=True)
-    build.ninja(ctx.output_dir, "fuzzers")
+    build.gn_gen(ctx, pw_toolchain_OSS_FUZZ_ENABLED=True)
+    build.ninja(ctx, "fuzzers")
 
 
 def _run_cmake(ctx: PresubmitContext, toolchain='host_clang') -> None:
@@ -297,8 +356,7 @@ def _run_cmake(ctx: PresubmitContext, toolchain='host_clang') -> None:
         env = build.env_with_clang_vars()
 
     toolchain_path = ctx.root / 'pw_toolchain' / toolchain / 'toolchain.cmake'
-    build.cmake(ctx.root,
-                ctx.output_dir,
+    build.cmake(ctx,
                 f'-DCMAKE_TOOLCHAIN_FILE={toolchain_path}',
                 '-DCMAKE_EXPORT_COMPILE_COMMANDS=1',
                 f'-Ddir_pw_third_party_nanopb={ctx.package_root / "nanopb"}',
@@ -310,131 +368,101 @@ def _run_cmake(ctx: PresubmitContext, toolchain='host_clang') -> None:
                         'CMakeLists.txt'))
 def cmake_clang(ctx: PresubmitContext):
     _run_cmake(ctx, toolchain='host_clang')
-    build.ninja(ctx.output_dir, 'pw_apps', 'pw_run_tests.modules')
+    build.ninja(ctx, 'pw_apps', 'pw_run_tests.modules')
 
 
 @filter_paths(endswith=(*format_code.C_FORMAT.extensions, '.cmake',
                         'CMakeLists.txt'))
 def cmake_gcc(ctx: PresubmitContext):
     _run_cmake(ctx, toolchain='host_gcc')
-    build.ninja(ctx.output_dir, 'pw_apps', 'pw_run_tests.modules')
+    build.ninja(ctx, 'pw_apps', 'pw_run_tests.modules')
 
 
-# TODO(pwbug/180): Slowly add modules here that work with bazel until all
-# modules are added. Then replace with //...
-_MODULES_THAT_BUILD_WITH_BAZEL = [
-    '//pw_allocator/...',
-    '//pw_analog/...',
-    '//pw_assert/...',
-    '//pw_assert_basic/...',
-    '//pw_assert_log/...',
-    '//pw_base64/...',
-    '//pw_bloat/...',
-    '//pw_build/...',
-    '//pw_checksum/...',
-    '//pw_chrono_embos/...',
-    '//pw_chrono_freertos/...',
-    '//pw_chrono_stl/...',
-    '//pw_chrono_threadx/...',
-    '//pw_cli/...',
-    '//pw_containers/...',
-    '//pw_cpu_exception/...',
-    '//pw_docgen/...',
-    '//pw_doctor/...',
-    '//pw_env_setup/...',
-    '//pw_fuzzer/...',
-    '//pw_hdlc/java/...',
-    '//pw_hex_dump/...',
-    '//pw_i2c/...',
-    '//pw_interrupt/...',
-    '//pw_interrupt_cortex_m/...',
-    '//pw_libc/...',
-    '//pw_log/...',
-    '//pw_log_basic/...',
-    '//pw_malloc/...',
-    '//pw_malloc_freelist/...',
-    '//pw_multisink/...',
-    '//pw_polyfill/...',
-    '//pw_preprocessor/...',
-    '//pw_protobuf/...',
-    '//pw_protobuf_compiler/...',
-    '//pw_random/...',
-    '//pw_result/...',
-    '//pw_rpc/...',
-    '//pw_span/...',
-    '//pw_status/...',
-    '//pw_stream/...',
-    '//pw_string/...',
-    '//pw_sync_baremetal/...',
-    '//pw_sync_embos/...',
-    '//pw_sync_freertos/...',
-    '//pw_sync_stl/...',
-    '//pw_sync_threadx/...',
-    '//pw_sys_io/...',
-    '//pw_sys_io_baremetal_lm3s6965evb/...',
-    '//pw_sys_io_baremetal_stm32f429/...',
-    '//pw_sys_io_stdio/...',
-    '//pw_thread/...',
-    '//pw_thread_stl/...',
-    '//pw_tokenizer/ts/...',
-    '//pw_tool/...',
-    '//pw_toolchain/...',
-    '//pw_transfer/...',
-    '//pw_unit_test/...',
-    '//pw_varint/...',
-    '//pw_web_ui/...',
-]
+# TODO(b/235882003): Slowly remove targets from here that work with bazel until
+# none remain.
+_TARGETS_THAT_DO_NOT_BUILD_WITH_BAZEL = (
+    '-//pw_arduino_build',
+    '-//pw_blob_store/...:all',
+    '-//pw_boot/...:all',
+    '-//pw_cpu_exception_cortex_m/...:all',
+    '-//pw_crypto/...:all',  # TODO(b/236321905) Remove when passing.
+    '-//pw_file/...:all',
+    '-//pw_function:function_test',  # TODO(b/241821115) Remove when passing.
+    '-//pw_hdlc/rpc_example',  # TODO(b/241575924) Remove when passing.
+    '-//pw_i2c_mcuxpresso/...:all',
+    '-//pw_kvs/...:all',
+    '-//pw_log:log_proto_py_pb2',  # TODO(b/241456982) Remove when passing.
+    '-//pw_log:log_proto_py_pb2_genproto',
+    '-//pw_log_null/...:all',
+    '-//pw_log_string/...:all',
+    '-//pw_metric/...:all',
+    '-//pw_minimal_cpp_stdlib/...:all',
+    '-//pw_persistent_ram/...:all',
+    '-//pw_snapshot/py/...:all',
+    '-//pw_snapshot:metadata_proto_py_pb2',
+    '-//pw_snapshot:metadata_proto_py_pb2_genproto',
+    '-//pw_snapshot:snapshot_proto_py_pb2',
+    '-//pw_snapshot:snapshot_proto_py_pb2_genproto',
+    # TODO(b/232427554): Get pw_software_update to build.
+    '-//pw_software_update:bundled_update_py_pb2',
+    '-//pw_software_update:bundled_update_py_pb2_genproto',
+    '-//pw_software_update:bundled_update_service',
+    '-//pw_software_update:bundled_update_service_test',
+    '-//pw_software_update:bundled_update_service_pwpb',
+    '-//pw_software_update:bundled_update_service_pwpb_test',
+    '-//pw_software_update:update_bundle',
+    '-//pw_software_update:update_bundle_test',
+    '-//pw_spi/...:all',
+    '-//pw_sys_io_arduino/...:all',
+    '-//pw_sys_io_mcuxpresso/...:all',
+    '-//pw_sys_io_stm32cube/...:all',
+    '-//pw_system/...:all',
+    '-//pw_thread/py/...:all',
+    '-//pw_thread:thread_proto_py_pb2',
+    '-//pw_thread:thread_proto_py_pb2_genproto',
+    '-//pw_thread_embos/...:all',
+    '-//pw_thread_freertos/...:all',
+    '-//pw_thread_threadx/...:all',
+    '-//pw_tls_client/...:all',
+    '-//pw_tls_client_boringssl/...:all',
+    '-//pw_tls_client_mbedtls/...:all',
+    # TODO(b/241456982) Remove when passing.
+    '-//pw_tokenizer:tokenizer_proto_py_pb2',
+    '-//pw_tokenizer:tokenizer_proto_py_pb2_genproto',
+    '-//pw_trace/...:all',
+    '-//pw_trace_tokenized/...:all',
+    '-//pw_work_queue/...:all',
+    '-//targets/arduino/...:all',
+    '-//targets/emcraft_sf2_som/...:all',
+    '-//targets/lm3s6965evb_qemu/...:all',
+    '-//targets/mimxrt595_evk/...:all',
+    '-//targets/stm32f429i_disc1/...:all',
+    '-//targets/stm32f429i_disc1_stm32cube/...:all',
+    '-//targets/rp2040/...:all',
+    '-//third_party/boringssl/...:all',
+    '-//third_party/micro_ecc/...:all',
+)
 
-# TODO(pwbug/180): Slowly add modules here that work with bazel until all
-# modules are added. Then replace with //...
-_MODULES_THAT_TEST_WITH_BAZEL = [
-    '//pw_allocator/...',
-    '//pw_analog/...',
-    '//pw_assert/...',
-    '//pw_base64/...',
-    '//pw_checksum/...',
-    '//pw_cli/...',
-    '//pw_containers/...',
-    '//pw_hdlc/java/...',
-    '//pw_hex_dump/...',
-    '//pw_i2c/...',
-    '//pw_libc/...',
-    '//pw_log/...',
-    '//pw_multisink/...',
-    '//pw_polyfill/...',
-    '//pw_preprocessor/...',
-    '//pw_protobuf/...',
-    '//pw_protobuf_compiler/...',
-    '//pw_random/...',
-    '//pw_result/...',
-    '//pw_rpc/...',
-    '//pw_span/...',
-    '//pw_status/...',
-    '//pw_stream/...',
-    '//pw_string/...',
-    '//pw_thread/...',
-    '//pw_thread_stl/...',
-    '//pw_tokenizer/ts/...',
-    '//pw_transfer/...',
-    '//pw_unit_test/...',
-    '//pw_varint/...',
-    '//:buildifier_test',
-]
+# TODO(b/235882003): Slowly remove targets from here that work with bazel until
+# none remain.
+_TARGETS_THAT_DO_NOT_TEST_WITH_BAZEL = _TARGETS_THAT_DO_NOT_BUILD_WITH_BAZEL + (
+    '-//pw_malloc_freelist/...:all', )
 
 
 @filter_paths(endswith=(*format_code.C_FORMAT.extensions, '.bazel', '.bzl',
                         'BUILD'))
 def bazel_test(ctx: PresubmitContext) -> None:
     """Runs bazel test on each bazel compatible module"""
-    build.bazel(ctx, 'test', *_MODULES_THAT_TEST_WITH_BAZEL,
-                '--test_output=errors')
+    build.bazel(ctx, 'test', '--test_output=errors', '--', '//...',
+                *_TARGETS_THAT_DO_NOT_TEST_WITH_BAZEL)
 
 
 @filter_paths(endswith=(*format_code.C_FORMAT.extensions, '.bazel', '.bzl',
                         'BUILD'))
 def bazel_build(ctx: PresubmitContext) -> None:
     """Runs Bazel build on each Bazel compatible module."""
-    build.bazel(ctx, 'build', *_MODULES_THAT_BUILD_WITH_BAZEL)
+    build.bazel(ctx, 'build', '--', '//...',
+                *_TARGETS_THAT_DO_NOT_BUILD_WITH_BAZEL)
 
 
 def pw_transfer_integration_test(ctx: PresubmitContext) -> None:
@@ -499,39 +527,6 @@ def edit_compile_commands(in_path: Path, out_path: Path,
         json.dump(compile_commands, out_file, indent=2)
 
 
-# The first line must be regex because of the '20\d\d' date
-COPYRIGHT_FIRST_LINE = r'Copyright 20\d\d The Pigweed Authors'
-COPYRIGHT_COMMENTS = r'(#|//| \*|REM|::)'
-COPYRIGHT_BLOCK_COMMENTS = (
-    # HTML comments
-    (r'<!--', r'-->'),
-    # Jinja comments
-    (r'{#', r'#}'),
-)
-
-COPYRIGHT_FIRST_LINE_EXCEPTIONS = (
-    '#!',
-    '/*',
-    '@echo off',
-    '# -*-',
-    ':',
-)
-
-COPYRIGHT_LINES = tuple("""\
-
-Licensed under the Apache License, Version 2.0 (the "License"); you may not
-use this file except in compliance with the License. You may obtain a copy of
-the License at
-
-    https://www.apache.org/licenses/LICENSE-2.0
-
-Unless required by applicable law or agreed to in writing, software
-distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
-WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
-License for the specific language governing permissions and limitations under
-the License.
-""".splitlines())
-
 _EXCLUDE_FROM_COPYRIGHT_NOTICE: Sequence[str] = (
     # Configuration
     r'^(?:.+/)?\..+$',
@@ -547,7 +542,10 @@ _EXCLUDE_FROM_COPYRIGHT_NOTICE: Sequence[str] = (
     r'\bgo.(mod|sum)$',
     r'\bpackage.json$',
     r'\byarn.lock$',
+    r'\bpackage-lock.json$',
     # Data files
+    r'\.bin$',
+    r'\.csv$',
     r'\.elf$',
     r'\.gif$',
     r'\.jpg$',
@@ -567,47 +565,52 @@ _EXCLUDE_FROM_COPYRIGHT_NOTICE: Sequence[str] = (
     r'\.patch$',
 )
 
+# Regular expression for the copyright comment. "\1" refers to the comment
+# characters and "\2" refers to space after the comment characters, if any.
+# All period characters are escaped using a replace call.
+_COPYRIGHT = re.compile(
+r"""(#|//|::| \*|)( ?)Copyright 2\d{3} The Pigweed Authors
+\1
+\1\2Licensed under the Apache License, Version 2.0 \(the "License"\); you may not
+\1\2use this file except in compliance with the License. You may obtain a copy of
+\1\2the License at
+\1
+\1(?:\2    |\t)https://www.apache.org/licenses/LICENSE-2.0
+\1
+\1\2Unless required by applicable law or agreed to in writing, software
+\1\2distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+\1\2WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+\1\2License for the specific language governing permissions and limitations under
+\1\2the License.
+""".replace('.', r'\.'), re.MULTILINE)  # pylint: disable=line-too-long # yapf: disable
 
-def match_block_comment_start(line: str) -> Optional[str]:
-    """Matches the start of a block comment and returns the end."""
-    for block_comment in COPYRIGHT_BLOCK_COMMENTS:
-        if re.match(block_comment[0], line):
-            # Return the end of the block comment
-            return block_comment[1]
-    return None
+_SKIP_LINE_PREFIXES = (
+    '#!',
+    '@echo off',
+    ':<<',
+    '/*',
+    ' * @jest-environment jsdom',
+    ' */',
+    '{#',  # Jinja comment block
+    '# -*- coding: utf-8 -*-',
+    '<!--',
+)
 
 
-def copyright_read_first_line(
-        file: IO) -> Tuple[Optional[str], Optional[str], Optional[str]]:
-    """Reads the file until it reads a valid first copyright line.
+def _read_notice_lines(file: TextIO) -> Iterable[str]:
+    lines = iter(file)
+    try:
+        # Read until the first line of the copyright notice.
+        line = next(lines)
+        while line.isspace() or line.startswith(_SKIP_LINE_PREFIXES):
+            line = next(lines)
 
-    Returns (comment, block_comment, line). comment and block_comment are
-    mutually exclusive and refer to the comment character sequence and whether
-    they form a block comment or a line comment. line is the first line of
-    the copyright, and is used for error reporting.
-    """
-    line = file.readline()
-    first_line_matcher = re.compile(COPYRIGHT_COMMENTS + ' ' +
-                                    COPYRIGHT_FIRST_LINE)
-    while line:
-        end_block_comment = match_block_comment_start(line)
-        if end_block_comment:
-            next_line = file.readline()
-            copyright_line = re.match(COPYRIGHT_FIRST_LINE, next_line)
-            if not copyright_line:
-                return (None, None, line)
-            return (None, end_block_comment, line)
+        yield line
 
-        first_line = first_line_matcher.match(line)
-        if first_line:
-            return (first_line.group(1), None, line)
-
-        if (line.strip()
-                and not line.startswith(COPYRIGHT_FIRST_LINE_EXCEPTIONS)):
-            return (None, None, line)
-
-        line = file.readline()
-    return (None, None, None)
+        for _ in range(12):  # The notice is 13 lines; read the remaining 12.
+            yield next(lines)
+    except StopIteration:
+        return
 
 
 @filter_paths(exclude=_EXCLUDE_FROM_COPYRIGHT_NOTICE)
@@ -616,43 +619,12 @@ def copyright_notice(ctx: PresubmitContext):
     errors = []
 
     for path in ctx.paths:
-
         if path.stat().st_size == 0:
             continue  # Skip empty files
 
-        if path.is_dir():
-            continue  # Skip submodules which are included in ctx.paths.
-
         with path.open() as file:
-            (comment, end_block_comment,
-             line) = copyright_read_first_line(file)
-
-            if not line:
-                _LOG.warning('%s: invalid first line', path)
+            if not _COPYRIGHT.match(''.join(_read_notice_lines(file))):
                 errors.append(path)
-                continue
-
-            if not (comment or end_block_comment):
-                _LOG.warning('%s: invalid first line %r', path, line)
-                errors.append(path)
-                continue
-
-            if end_block_comment:
-                expected_lines = COPYRIGHT_LINES + (end_block_comment, )
-            else:
-                expected_lines = COPYRIGHT_LINES
-
-            for expected, actual in zip(expected_lines, file):
-                if end_block_comment:
-                    expected_line = expected + '\n'
-                elif comment:
-                    expected_line = (comment + ' ' + expected).rstrip() + '\n'
-
-                if expected_line != actual:
-                    _LOG.warning('  bad line: %r', actual)
-                    _LOG.warning('  expected: %r', expected_line)
-                    errors.append(path)
-                    break
 
     if errors:
         _LOG.warning('%s with a missing or incorrect copyright notice:\n%s',
@@ -664,9 +636,14 @@ _BAZEL_SOURCES_IN_BUILD = tuple(format_code.C_FORMAT.extensions)
 _GN_SOURCES_IN_BUILD = ('setup.cfg', '.toml', '.rst', '.py',
                         *_BAZEL_SOURCES_IN_BUILD)
 
+SOURCE_FILES_FILTER = presubmit.FileFilter(endswith=_GN_SOURCES_IN_BUILD,
+                                           suffix=('.bazel', '.bzl', '.gn',
+                                                   '.gni'),
+                                           exclude=(r'zephyr.*/',
+                                                    r'android.*/'))
 
-@filter_paths(endswith=(*_GN_SOURCES_IN_BUILD, 'BUILD', '.bzl', '.gn', '.gni'),
-              exclude=['zephyr.*/', 'android.*/'])
+
+@SOURCE_FILES_FILTER.apply_to_check()
 def source_is_in_build_files(ctx: PresubmitContext):
     """Checks that source files are in the GN and Bazel builds."""
     missing = build.check_builds_for_files(
@@ -721,6 +698,10 @@ def commit_message_format(_: PresubmitContext):
     for line in lines:
         _LOG.debug(line)
 
+    if not lines:
+        _LOG.error('The commit message is too short!')
+        raise PresubmitFailure
+
     # Ignore Gerrit-generated reverts.
     if ('Revert' in lines[0]
             and 'This reverts commit ' in git_repo.commit_message()
@@ -728,10 +709,6 @@ def commit_message_format(_: PresubmitContext):
         _LOG.warning('Ignoring apparent Gerrit-generated revert')
         return
 
-    if not lines:
-        _LOG.error('The commit message is too short!')
-        raise PresubmitFailure
-
     errors = 0
 
     if len(lines[0]) > 72:
@@ -753,6 +730,13 @@ def commit_message_format(_: PresubmitContext):
                      lines[1])
         errors += 1
 
+    # Ignore the line length check for Copybara imports so they can include the
+    # commit hash and description for imported commits.
+    if not errors and ('Copybara import' in lines[0]
+                       and 'GitOrigin-RevId:' in git_repo.commit_message()):
+        _LOG.warning('Ignoring Copybara import')
+        return
+
     # Check that the lines are 72 characters or less, but skip any lines that
     # might possibly have a URL, path, or metadata in them. Also skip any lines
     # with non-ASCII characters.
@@ -775,8 +759,8 @@ def commit_message_format(_: PresubmitContext):
 @filter_paths(endswith=(*format_code.C_FORMAT.extensions, '.py'))
 def static_analysis(ctx: PresubmitContext):
     """Runs all available static analysis tools."""
-    build.gn_gen(ctx.root, ctx.output_dir)
-    build.ninja(ctx.output_dir, 'python.lint', 'static_analysis')
+    build.gn_gen(ctx)
+    build.ninja(ctx, 'python.lint', 'static_analysis')
 
 
 def renode_check(ctx: PresubmitContext):
@@ -789,41 +773,65 @@ def renode_check(ctx: PresubmitContext):
 #
 
 OTHER_CHECKS = (
-    cpp_checks.all_sanitizers(),
     # Build that attempts to duplicate the build OSS-Fuzz does. Currently
     # failing.
     oss_fuzz_build,
-    # TODO(pwbug/346): Enable all Bazel tests when they're fixed.
+    # TODO(b/235277910): Enable all Bazel tests when they're fixed.
     bazel_test,
     cmake_clang,
     cmake_gcc,
     gn_boringssl_build,
     build.gn_gen_check,
-    gn_nanopb_build,
-    gn_crypto_mbedtls_build,
-    gn_crypto_boringssl_build,
-    gn_crypto_micro_ecc_build,
-    gn_software_update_build,
     gn_full_build_check,
     gn_full_qemu_check,
+    gn_combined_build_check,
     gn_clang_build,
     gn_gcc_build,
-    gn_pw_system_demo_build,
     pw_transfer_integration_test,
     renode_check,
     static_analysis,
     stm32f429i,
+    npm_presubmit.npm_test,
+    todo_check.create(todo_check.BUGS_OR_USERNAMES),
 )
 
+# The misc program differs from other_checks in that checks in the misc
+# program block CQ on Linux.
+MISC = (
+    gn_nanopb_build,
+    gn_pw_system_demo_build,
+    gn_teensy_build,
+)
+
+SANITIZERS = (cpp_checks.all_sanitizers(), )
+
+# TODO(b/243380637) Merge into SECURITY.
+CRYPTO = (
+    gn_crypto_mbedtls_build,
+    gn_crypto_boringssl_build,
+    gn_crypto_micro_ecc_build,
+)
+
+SECURITY = (
+    CRYPTO,
+    gn_software_update_build,
+)
+
+# Avoid running all checks on specific paths.
+PATH_EXCLUSIONS = (re.compile(r'\bthird_party/fuchsia/repo/'), )
+
 _LINTFORMAT = (
     commit_message_format,
     copyright_notice,
     format_code.presubmit_checks(),
-    inclusive_language.inclusive_language.with_filter(
-        exclude=(r'\byarn.lock$', )),
+    inclusive_language.inclusive_language.with_filter(exclude=(
+        r'\byarn.lock$',
+        r'\bpackage-lock.json$',
+    )),
     cpp_checks.pragma_once,
     build.bazel_lint,
     source_is_in_build_files,
+    shell_checks.shellcheck if shutil.which('shellcheck') else (),
 )
 
 LINTFORMAT = (
@@ -835,7 +843,7 @@ LINTFORMAT = (
 QUICK = (
     _LINTFORMAT,
     gn_quick_build_check,
-    # TODO(pwbug/141): Re-enable CMake and Bazel for Mac after we have fixed the
+    # TODO(b/34884583): Re-enable CMake and Bazel for Mac after we have fixed
     # the clang issues. The problem is that all clang++ invocations need the
     # two extra flags: "-nostdc++" and "${clang_prefix}/../lib/libc++.a".
     cmake_clang if sys.platform != 'darwin' else (),
@@ -868,8 +876,12 @@ FULL = (
 PROGRAMS = Programs(
     full=FULL,
     lintformat=LINTFORMAT,
+    misc=MISC,
     other_checks=OTHER_CHECKS,
     quick=QUICK,
+    crypto=CRYPTO,
+    sanitizers=SANITIZERS,
+    security=SECURITY,
 )
 
 
@@ -886,18 +898,18 @@ def parse_args() -> argparse.Namespace:
     return parser.parse_args()
 
 
-def run(install: bool, **presubmit_args) -> int:
+def run(install: bool, exclude: list, **presubmit_args) -> int:
     """Entry point for presubmit."""
 
     if install:
-        # TODO(pwbug/209, pwbug/386) inclusive-language: disable
-        install_hook(__file__, 'pre-push',
-                     ['--base', 'origin/master..HEAD', '--program', 'quick'],
-                     Path.cwd())
-        # TODO(pwbug/209, pwbug/386) inclusive-language: enable
+        install_git_hook('pre-push', [
+            'python', '-m', 'pw_presubmit.pigweed_presubmit', '--base',
+            'origin/main..HEAD', '--program', 'quick'
+        ])
         return 0
 
-    return cli.run(**presubmit_args)
+    exclude.extend(PATH_EXCLUSIONS)
+    return cli.run(exclude=exclude, **presubmit_args)
 
 
 def main() -> int:
