@@ -52,22 +52,16 @@ import re
 import subprocess
 import time
 from typing import (Callable, Collection, Dict, Iterable, Iterator, List,
-                    NamedTuple, Optional, Pattern, Sequence, Set, Tuple, Union)
+                    Optional, Pattern, Sequence, Set, Tuple, Union)
 
+import pw_cli.color
 import pw_cli.env
 from pw_presubmit import git_repo, tools
 from pw_presubmit.tools import plural
 
 _LOG: logging.Logger = logging.getLogger(__name__)
 
-color_red = tools.make_color(31)
-color_bold_red = tools.make_color(31, 1)
-color_black_on_red = tools.make_color(30, 41)
-color_yellow = tools.make_color(33, 1)
-color_green = tools.make_color(32)
-color_black_on_green = tools.make_color(30, 42)
-color_aqua = tools.make_color(36)
-color_bold_white = tools.make_color(37, 1)
+_COLOR = pw_cli.color.colors()
 
 _SUMMARY_BOX = '══╦╗ ║║══╩╝'
 _CHECK_UPPER = '━━━┓       '
@@ -104,7 +98,7 @@ def _box(style, left, middle, right, box=tools.make_box('><>')) -> str:
 
 class PresubmitFailure(Exception):
     """Optional exception to use for presubmit failures."""
-    def __init__(self, description: str = '', path=None):
+    def __init__(self, description: str = '', path: Path = None):
         super().__init__(f'{path}: {description}' if path else description)
 
 
@@ -116,11 +110,11 @@ class _Result(enum.Enum):
 
     def colorized(self, width: int, invert: bool = False) -> str:
         if self is _Result.PASS:
-            color = color_black_on_green if invert else color_green
+            color = _COLOR.black_on_green if invert else _COLOR.green
         elif self is _Result.FAIL:
-            color = color_black_on_red if invert else color_red
+            color = _COLOR.black_on_red if invert else _COLOR.red
         elif self is _Result.CANCEL:
-            color = color_yellow
+            color = _COLOR.yellow
         else:
             color = lambda value: value
 
@@ -176,7 +170,7 @@ class Programs(collections.abc.Mapping):
         return len(self._programs)
 
 
-@dataclasses.dataclass(frozen=True)
+@dataclasses.dataclass
 class PresubmitContext:
     """Context passed into presubmit checks."""
     root: Path
@@ -184,27 +178,90 @@ class PresubmitContext:
     output_dir: Path
     paths: Tuple[Path, ...]
     package_root: Path
+    # TODO(b/233808334): Remove default value after updating downstream
+    # projects to use this new value.
+    override_gn_args: Dict[str, str] = dataclasses.field(default_factory=dict)
+    _failed: bool = False
 
-    def relative_paths(self, start: Optional[Path] = None) -> Tuple[Path, ...]:
-        return tuple(
-            tools.relative_paths(self.paths, start if start else self.root))
+    @property
+    def failed(self) -> bool:
+        return self._failed
 
-    def paths_by_repo(self) -> Dict[Path, List[Path]]:
-        repos = collections.defaultdict(list)
+    def fail(self, description: str, path: Path = None):
+        """Add a failure to this presubmit step.
 
-        for path in self.paths:
-            repos[git_repo.root(path)].append(path)
-
-        return repos
+        If this is called at least once the step fails, but not immediately—the
+        check is free to continue and possibly call this method again.
+        """
+        _LOG.warning('%s', PresubmitFailure(description, path))
+        self._failed = True
 
 
-class _Filter(NamedTuple):
-    endswith: Tuple[str, ...] = ('', )
-    exclude: Tuple[Pattern[str], ...] = ()
+class FileFilter:
+    """Allows checking if a path matches a series of filters.
 
-    def matches(self, path: str) -> bool:
-        return (any(path.endswith(end) for end in self.endswith)
-                and not any(exp.search(path) for exp in self.exclude))
+    Positive filters (e.g. the file name matches a regex) and negative filters
+    (path does not match a regular expression) may be applied.
+    """
+
+    _StrOrPattern = Union[Pattern, str]
+
+    def __init__(
+        self,
+        *,
+        exclude: Iterable[_StrOrPattern] = (),
+        endswith: Iterable[str] = (),
+        name: Iterable[_StrOrPattern] = (),
+        suffix: Iterable[str] = ()
+    ) -> None:
+        """Creates a FileFilter with the provided filters.
+
+        Args:
+            endswith: True if the end of the path is equal to any of the passed
+                      strings
+            exclude: If any of the passed regular expresion match return False.
+                     This overrides and other matches.
+            name: Regexs to match with file names(pathlib.Path.name). True if
+                  the resulting regex matches the entire file name.
+            suffix: True if final suffix (as determined by pathlib.Path) is
+                    matched by any of the passed str.
+        """
+        self.exclude = tuple(re.compile(i) for i in exclude)
+
+        self.endswith = tuple(endswith)
+        self.name = tuple(re.compile(i) for i in name)
+        self.suffix = tuple(suffix)
+
+    def matches(self, path: Union[str, Path]) -> bool:
+        """Returns true if the path matches any filter but not an exclude.
+
+        If no positive filters are specified, any paths that do not match a
+        negative filter are considered to match.
+
+        If 'path' is a Path object it is rendered as a posix path (i.e.
+        using "/" as the path seperator) before testing with 'exclude' and
+        'endswith'.
+        """
+
+        posix_path = path.as_posix() if isinstance(path, Path) else path
+        if any(bool(exp.search(posix_path)) for exp in self.exclude):
+            return False
+
+        # If there are no positive filters set, accept all paths.
+        no_filters = not self.endswith and not self.name and not self.suffix
+
+        path_obj = Path(path)
+        return (no_filters or path_obj.suffix in self.suffix
+                or any(regex.fullmatch(path_obj.name) for regex in self.name)
+                or any(posix_path.endswith(end) for end in self.endswith))
+
+    def apply_to_check(self, always_run: bool = False) -> Callable:
+        def wrapper(func: Callable) -> Check:
+            return Check(check_function=func,
+                         path_filter=self,
+                         always_run=always_run)
+
+        return wrapper
 
 
 def _print_ui(*args) -> None:
@@ -216,7 +273,7 @@ class Presubmit:
     """Runs a series of presubmit checks on a list of files."""
     def __init__(self, root: Path, repos: Sequence[Path],
                  output_directory: Path, paths: Sequence[Path],
-                 package_root: Path):
+                 package_root: Path, override_gn_args: Dict[str, str]):
         self._root = root.resolve()
         self._repos = tuple(repos)
         self._output_directory = output_directory.resolve()
@@ -224,6 +281,7 @@ class Presubmit:
         self._relative_paths = tuple(
             tools.relative_paths(self._paths, self._root))
         self._package_root = package_root.resolve()
+        self._override_gn_args = override_gn_args
 
     def run(self, program: Program, keep_going: bool = False) -> bool:
         """Executes a series of presubmit checks on the paths."""
@@ -242,7 +300,7 @@ class Presubmit:
         _print_ui()
 
         if not self._paths:
-            _print_ui(color_yellow('No files are being checked!'))
+            _print_ui(_COLOR.yellow('No files are being checked!'))
 
         _LOG.debug('Checks:\n%s', '\n'.join(c.name for c, _ in checks))
 
@@ -257,7 +315,7 @@ class Presubmit:
             program: Sequence[Callable]) -> List[Tuple[Check, Sequence[Path]]]:
         """Returns list of (check, paths) for checks that should run."""
         checks = [c if isinstance(c, Check) else Check(c) for c in program]
-        filter_to_checks: Dict[_Filter,
+        filter_to_checks: Dict[FileFilter,
                                List[Check]] = collections.defaultdict(list)
 
         for check in checks:
@@ -267,7 +325,7 @@ class Presubmit:
         return [(c, check_to_paths[c]) for c in checks if c in check_to_paths]
 
     def _map_checks_to_paths(
-        self, filter_to_checks: Dict[_Filter, List[Check]]
+        self, filter_to_checks: Dict[FileFilter, List[Check]]
     ) -> Dict[Check, Sequence[Path]]:
         checks_to_paths: Dict[Check, Sequence[Path]] = {}
 
@@ -332,6 +390,7 @@ class Presubmit:
                 output_dir=output_directory,
                 paths=paths,
                 package_root=self._package_root,
+                override_gn_args=self._override_gn_args,
             )
 
         finally:
@@ -389,7 +448,8 @@ def _process_pathspecs(repos: Iterable[Path],
     return pathspecs_by_repo
 
 
-def run(program: Sequence[Callable],
+def run(  # pylint: disable=too-many-arguments
+        program: Sequence[Callable],
         root: Path,
         repos: Collection[Path] = (),
         base: Optional[str] = None,
@@ -398,6 +458,7 @@ def run(program: Sequence[Callable],
         output_directory: Optional[Path] = None,
         package_root: Path = None,
         only_list_steps: bool = False,
+        override_gn_args: Sequence[Tuple[str, str]] = (),
         keep_going: bool = False) -> bool:
     """Lists files in the current Git repo and runs a Presubmit with them.
 
@@ -422,6 +483,7 @@ def run(program: Sequence[Callable],
         output_directory: where to place output files
         package_root: where to place package files
         only_list_steps: print step names instead of running them
+        override_gn_args: additional GN args to set on steps
         keep_going: whether to continue running checks if an error occurs
 
     Returns:
@@ -429,10 +491,15 @@ def run(program: Sequence[Callable],
     """
     repos = [repo.resolve() for repo in repos]
 
+    non_empty_repos = []
     for repo in repos:
-        if git_repo.root(repo) != repo:
-            raise ValueError(f'{repo} is not the root of a Git repo; '
-                             'presubmit checks must be run from a Git repo')
+        if list(repo.iterdir()):
+            non_empty_repos.append(repo)
+            if git_repo.root(repo) != repo:
+                raise ValueError(
+                    f'{repo} is not the root of a Git repo; '
+                    'presubmit checks must be run from a Git repo')
+    repos = non_empty_repos
 
     pathspecs_by_repo = _process_pathspecs(repos, paths)
 
@@ -459,6 +526,7 @@ def run(program: Sequence[Callable],
         output_directory=output_directory,
         paths=files,
         package_root=package_root,
+        override_gn_args=dict(override_gn_args or {}),
     )
 
     if only_list_steps:
@@ -484,12 +552,12 @@ class Check:
     """
     def __init__(self,
                  check_function: Callable,
-                 path_filter: _Filter = _Filter(),
-                 always_run: bool = True):
+                 path_filter: FileFilter = FileFilter(),
+                 always_run: bool = True) -> None:
         _ensure_is_valid_presubmit_check_function(check_function)
 
         self._check: Callable = check_function
-        self.filter: _Filter = path_filter
+        self.filter = path_filter
         self.always_run: bool = always_run
 
         # Since Check wraps a presubmit function, adopt that function's name.
@@ -498,16 +566,15 @@ class Check:
     def with_filter(
         self,
         *,
-        endswith: Iterable[str] = '',
+        endswith: Iterable[str] = (),
         exclude: Iterable[Union[Pattern[str], str]] = ()
     ) -> Check:
-        endswith = self.filter.endswith
-        if endswith:
-            endswith = endswith + _make_str_tuple(endswith)
-        exclude = self.filter.exclude + tuple(re.compile(e) for e in exclude)
+        return self.with_file_filter(
+            FileFilter(endswith=_make_str_tuple(endswith), exclude=exclude))
 
+    def with_file_filter(self, file_filter: FileFilter) -> Check:
         return Check(check_function=self._check,
-                     path_filter=_Filter(endswith=endswith, exclude=exclude),
+                     path_filter=file_filter,
                      always_run=self.always_run)
 
     @property
@@ -549,6 +616,9 @@ class Check:
             _print_ui()
             return _Result.CANCEL
 
+        if ctx.failed:
+            return _Result.FAIL
+
         return _Result.PASS
 
     def __call__(self, ctx: PresubmitContext, *args, **kwargs):
@@ -586,8 +656,10 @@ def _ensure_is_valid_presubmit_check_function(check: Callable) -> None:
              if required_args else ''))
 
 
-def filter_paths(endswith: Iterable[str] = '',
+def filter_paths(*,
+                 endswith: Iterable[str] = (),
                  exclude: Iterable[Union[Pattern[str], str]] = (),
+                 file_filter: FileFilter = None,
                  always_run: bool = False) -> Callable[[Callable], Check]:
     """Decorator for filtering the paths list for a presubmit check function.
 
@@ -599,15 +671,24 @@ def filter_paths(endswith: Iterable[str] = '',
     Args:
         endswith: str or iterable of path endings to include
         exclude: regular expressions of paths to exclude
-
+        file_filter: FileFilter used to select files
+        always_run: Run check even when no files match
     Returns:
         a wrapped version of the presubmit function
     """
+
+    if file_filter:
+        real_file_filter = file_filter
+        if endswith or exclude:
+            raise ValueError('Must specify either file_filter or '
+                             'endswith/exclude args, not both')
+    else:
+        # TODO(b/238426363): Remove these arguments and use FileFilter only.
+        real_file_filter = FileFilter(endswith=_make_str_tuple(endswith),
+                                      exclude=exclude)
+
     def filter_paths_for_function(function: Callable):
-        return Check(function,
-                     _Filter(_make_str_tuple(endswith),
-                             tuple(re.compile(e) for e in exclude)),
-                     always_run=always_run)
+        return Check(function, real_file_filter, always_run=always_run)
 
     return filter_paths_for_function
 
