@@ -14,7 +14,6 @@
 """This module defines the generated code for pw_protobuf C++ classes."""
 
 import abc
-from datetime import datetime
 import enum
 # Type ignore here for graphlib-backport on Python 3.8
 from graphlib import CycleError, TopologicalSorter  # type: ignore
@@ -257,7 +256,7 @@ class ReadMethod(ProtoMethod):
 
         Defined in subclasses.
 
-        e.g. 'uint32_t', 'std::span<std::byte>', etc.
+        e.g. 'uint32_t', 'pw::span<std::byte>', etc.
         """
         raise NotImplementedError()
 
@@ -307,7 +306,7 @@ class PackedReadMethod(ReadMethod):
         return '::pw::StatusWithSize'
 
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<{}>'.format(self._result_type()), 'out')]
+        return [('pw::span<{}>'.format(self._result_type()), 'out')]
 
 
 class PackedReadVectorMethod(ReadMethod):
@@ -329,7 +328,7 @@ class PackedReadVectorMethod(ReadMethod):
 class MessageProperty(ProtoMember):
     """Base class for a C++ property for a field in a protobuf message."""
     def name(self) -> str:
-        return self._field.enum_name().lower()
+        return self._field.field_name()
 
     def should_appear(self) -> bool:
         return True
@@ -356,6 +355,11 @@ class MessageProperty(ProtoMember):
         return options.use_callback or (self._field.is_repeated()
                                         and self.max_size() == 0)
 
+    def is_optional(self) -> bool:
+        """Returns whether the decoder should use std::optional."""
+        return (self._field.is_optional() and self.max_size() == 0
+                and self.wire_type() != 'kDelimited')
+
     def max_size(self) -> int:
         """Returns the maximum size of the field."""
         if self._field.is_repeated():
@@ -383,6 +387,11 @@ class MessageProperty(ProtoMember):
             return (f'{PROTOBUF_NAMESPACE}::Callback'
                     '<StreamEncoder, StreamDecoder>', self.name())
 
+        # Optional fields are wrapped in std::optional
+        if self.is_optional():
+            return ('std::optional<{}>'.format(self.type_name(from_root)),
+                    self.name())
+
         # Non-repeated fields have a member of just the type name.
         max_size = self.max_size()
         if max_size == 0:
@@ -419,6 +428,7 @@ class MessageProperty(ProtoMember):
             self._varint_type_table_entry(),
             'true' if self.fixed_size() else 'false',
             'true' if self._field.is_repeated() else 'false',
+            'true' if self.is_optional() else 'false',
             'true' if self.use_callback() else 'false',
             'offsetof(Message, {})'.format(self.name()),
             'sizeof(Message::{})'.format(self.name()),
@@ -587,7 +597,7 @@ class DoubleWriteMethod(WriteMethod):
 class PackedDoubleWriteMethod(PackedWriteMethod):
     """Method which writes a packed list of doubles."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const double>', 'values')]
+        return [('pw::span<const double>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedDouble'
@@ -653,7 +663,7 @@ class FloatWriteMethod(WriteMethod):
 class PackedFloatWriteMethod(PackedWriteMethod):
     """Method which writes a packed list of floats."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const float>', 'values')]
+        return [('pw::span<const float>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedFloat'
@@ -719,7 +729,7 @@ class Int32WriteMethod(WriteMethod):
 class PackedInt32WriteMethod(PackedWriteMethod):
     """Method which writes a packed list of int32."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const int32_t>', 'values')]
+        return [('pw::span<const int32_t>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedInt32'
@@ -788,7 +798,7 @@ class Sint32WriteMethod(WriteMethod):
 class PackedSint32WriteMethod(PackedWriteMethod):
     """Method which writes a packed list of sint32."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const int32_t>', 'values')]
+        return [('pw::span<const int32_t>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedSint32'
@@ -857,7 +867,7 @@ class Sfixed32WriteMethod(WriteMethod):
 class PackedSfixed32WriteMethod(PackedWriteMethod):
     """Method which writes a packed list of sfixed32."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const int32_t>', 'values')]
+        return [('pw::span<const int32_t>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedSfixed32'
@@ -923,7 +933,7 @@ class Int64WriteMethod(WriteMethod):
 class PackedInt64WriteMethod(PackedWriteMethod):
     """Method which writes a packed list of int64."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const int64_t>', 'values')]
+        return [('pw::span<const int64_t>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedInt64'
@@ -992,7 +1002,7 @@ class Sint64WriteMethod(WriteMethod):
 class PackedSint64WriteMethod(PackedWriteMethod):
     """Method which writes a packst list of sint64."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const int64_t>', 'values')]
+        return [('pw::span<const int64_t>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedSint64'
@@ -1061,7 +1071,7 @@ class Sfixed64WriteMethod(WriteMethod):
 class PackedSfixed64WriteMethod(PackedWriteMethod):
     """Method which writes a packed list of sfixed64."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const int64_t>', 'values')]
+        return [('pw::span<const int64_t>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedSfixed4'
@@ -1127,7 +1137,7 @@ class Uint32WriteMethod(WriteMethod):
 class PackedUint32WriteMethod(PackedWriteMethod):
     """Method which writes a packed list of uint32."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const uint32_t>', 'values')]
+        return [('pw::span<const uint32_t>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedUint32'
@@ -1196,7 +1206,7 @@ class Fixed32WriteMethod(WriteMethod):
 class PackedFixed32WriteMethod(PackedWriteMethod):
     """Method which writes a packed list of fixed32."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const uint32_t>', 'values')]
+        return [('pw::span<const uint32_t>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedFixed32'
@@ -1262,7 +1272,7 @@ class Uint64WriteMethod(WriteMethod):
 class PackedUint64WriteMethod(PackedWriteMethod):
     """Method which writes a packed list of uint64."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const uint64_t>', 'values')]
+        return [('pw::span<const uint64_t>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedUint64'
@@ -1331,7 +1341,7 @@ class Fixed64WriteMethod(WriteMethod):
 class PackedFixed64WriteMethod(PackedWriteMethod):
     """Method which writes a packed list of fixed64."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const uint64_t>', 'values')]
+        return [('pw::span<const uint64_t>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedFixed64'
@@ -1397,7 +1407,7 @@ class BoolWriteMethod(WriteMethod):
 class PackedBoolWriteMethod(PackedWriteMethod):
     """Method which writes a packed list of bools."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const bool>', 'values')]
+        return [('pw::span<const bool>', 'values')]
 
     def _encoder_fn(self) -> str:
         return 'WritePackedBool'
@@ -1448,7 +1458,7 @@ class BoolProperty(MessageProperty):
 class BytesWriteMethod(WriteMethod):
     """Method which writes a proto bytes value."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const std::byte>', 'value')]
+        return [('pw::span<const std::byte>', 'value')]
 
     def _encoder_fn(self) -> str:
         return 'WriteBytes'
@@ -1460,7 +1470,7 @@ class BytesReadMethod(ReadMethod):
         return '::pw::StatusWithSize'
 
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<std::byte>', 'out')]
+        return [('pw::span<std::byte>', 'out')]
 
     def _decoder_fn(self) -> str:
         return 'ReadBytes'
@@ -1529,7 +1539,7 @@ class StringReadMethod(ReadMethod):
         return '::pw::StatusWithSize'
 
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<char>', 'out')]
+        return [('pw::span<char>', 'out')]
 
     def _decoder_fn(self) -> str:
         return 'ReadString'
@@ -1590,14 +1600,14 @@ class EnumWriteMethod(WriteMethod):
 class PackedEnumWriteMethod(PackedWriteMethod):
     """Method which writes a packed list of enum."""
     def params(self) -> List[Tuple[str, str]]:
-        return [('std::span<const {}>'.format(self._relative_type_namespace()),
+        return [('pw::span<const {}>'.format(self._relative_type_namespace()),
                  'values')]
 
     def body(self) -> List[str]:
         value_param = self.params()[0][1]
         line = (
             f'return {self._base_class}::WritePackedUint32('
-            f'{self.field_cast()}, std::span(reinterpret_cast<const uint32_t*>('
+            f'{self.field_cast()}, pw::span(reinterpret_cast<const uint32_t*>('
             f'{value_param}.data()), {value_param}.size()));')
         return [line]
 
@@ -1642,7 +1652,7 @@ class PackedEnumReadMethod(PackedReadMethod):
         value_param = self.params()[0][1]
         return [
             f'return ReadPackedUint32('
-            f'std::span(reinterpret_cast<uint32_t*>({value_param}.data()), '
+            f'pw::span(reinterpret_cast<uint32_t*>({value_param}.data()), '
             f'{value_param}.size()));'
         ]
 
@@ -1861,7 +1871,7 @@ def generate_class_for_message(message: ProtoMessage, root: ProtoNode,
             with output.indent():
                 output.write_line(
                     f'return {base_class}::Read('
-                    'std::as_writable_bytes(std::span(&message, 1)), '
+                    'pw::as_writable_bytes(pw::span(&message, 1)), '
                     'kMessageFields);')
             output.write_line('}')
         elif class_type in (ClassType.STREAMING_ENCODER,
@@ -1871,7 +1881,7 @@ def generate_class_for_message(message: ProtoMessage, root: ProtoNode,
             with output.indent():
                 output.write_line(
                     f'return {base_class}::Write('
-                    'std::as_bytes(std::span(&message, 1)), kMessageFields);')
+                    'pw::as_bytes(pw::span(&message, 1)), kMessageFields);')
             output.write_line('}')
 
         # Generate methods for each of the message's fields.
@@ -2102,21 +2112,34 @@ def generate_table_for_message(message: ProtoMessage, root: ProtoNode,
                           '{}::MessageField::kMaxFieldSize);'.format(
                               prop.name(), PROTOBUF_NAMESPACE))
 
-    output.write_line('inline constexpr {}::MessageField _kMessageFields[] '
-                      '= {{'.format(PROTOBUF_NAMESPACE, ))
+    # Zero-length C arrays are not permitted by the C++ standard, so only
+    # generate the message fields array if it is non-empty. Zero-length
+    # std::arrays are valid, but older toolchains may not support constexpr
+    # std::arrays, even with -std=c++17.
+    #
+    # The kMessageFields span is generated whether the message has fields or
+    # not. Only the span is referenced elsewhere.
+    if properties:
+        output.write_line(
+            f'inline constexpr {PROTOBUF_NAMESPACE}::MessageField '
+            ' _kMessageFields[] = {')
 
-    # Generate members for each of the message's fields.
-    with output.indent():
-        for prop in properties:
-            table = ', '.join(prop.table_entry())
-            output.write_line(f'{{{table}}},')
+        # Generate members for each of the message's fields.
+        with output.indent():
+            for prop in properties:
+                table = ', '.join(prop.table_entry())
+                output.write_line(f'{{{table}}},')
 
-    output.write_line('};')
-    output.write_line('PW_MODIFY_DIAGNOSTICS_POP();')
+        output.write_line('};')
+        output.write_line('PW_MODIFY_DIAGNOSTICS_POP();')
 
-    output.write_line('inline constexpr std::span<const {}::MessageField> '
-                      'kMessageFields = {{ _kMessageFields, {} }};'.format(
-                          PROTOBUF_NAMESPACE, len(properties)))
+        output.write_line(
+            f'inline constexpr pw::span<const {PROTOBUF_NAMESPACE}::'
+            'MessageField> kMessageFields = _kMessageFields;')
+    else:
+        output.write_line(
+            f'inline constexpr pw::span<const {PROTOBUF_NAMESPACE}::'
+            'MessageField> kMessageFields;')
 
     output.write_line(f'}}  // namespace {namespace}')
 
@@ -2208,13 +2231,12 @@ def generate_code_for_package(file_descriptor_proto, package: ProtoNode,
 
     output.write_line(f'// {os.path.basename(output.name())} automatically '
                       f'generated by {PLUGIN_NAME} {PLUGIN_VERSION}')
-    output.write_line(f'// on {datetime.now()}')
     output.write_line('#pragma once\n')
     output.write_line('#include <algorithm>')
     output.write_line('#include <array>')
     output.write_line('#include <cstddef>')
     output.write_line('#include <cstdint>')
-    output.write_line('#include <span>')
+    output.write_line('#include <optional>')
     output.write_line('#include <string_view>\n')
     output.write_line('#include "pw_assert/assert.h"')
     output.write_line('#include "pw_containers/vector.h"')
@@ -2224,6 +2246,7 @@ def generate_code_for_package(file_descriptor_proto, package: ProtoNode,
     output.write_line('#include "pw_protobuf/serialized_size.h"')
     output.write_line('#include "pw_protobuf/stream_decoder.h"')
     output.write_line('#include "pw_result/result.h"')
+    output.write_line('#include "pw_span/span.h"')
     output.write_line('#include "pw_status/status.h"')
     output.write_line('#include "pw_status/status_with_size.h"')
 
@@ -2291,7 +2314,7 @@ def process_proto_file(proto_file, proto_options) -> Iterable[OutputFile]:
     # message/enum nodes, then the second creates the fields in each. This is
     # done as non-primitive fields need pointers to their types, which requires
     # the entire tree to have been parsed into memory.
-    _, package_root = build_node_tree(proto_file, proto_options)
+    _, package_root = build_node_tree(proto_file, proto_options=proto_options)
 
     output_filename = _proto_filename_to_generated_header(proto_file.name)
     output_file = OutputFile(output_filename)
