@@ -16,7 +16,7 @@
 To start the console, provide a serial port as the --device argument and paths
 or globs for .proto files that define the RPC services to support:
 
-  python -m pw_hdlc.rpc_console --device /dev/ttyUSB0 sample.proto
+  python -m pw_system.console --device /dev/ttyUSB0 --proto-globs pw_rpc/echo.proto
 
 This starts an IPython console for communicating with the connected device. A
 few variables are predefined in the interactive console. These include:
@@ -29,7 +29,7 @@ few variables are predefined in the interactive console. These include:
 An example echo RPC command:
 
   rpcs.pw.rpc.EchoService.Echo(msg="hello!")
-"""
+"""  # pylint: disable=line-too-long
 
 import argparse
 import datetime
@@ -42,6 +42,7 @@ from types import ModuleType
 from typing import (
     Any,
     Collection,
+    Dict,
     Iterable,
     Iterator,
     List,
@@ -57,14 +58,19 @@ import pw_console.python_logging
 from pw_console import PwConsoleEmbed
 from pw_console.pyserial_wrapper import SerialWithLogging
 from pw_console.plugins.bandwidth_toolbar import BandwidthToolbar
-
+from pw_console.log_store import LogStore
 from pw_log.proto import log_pb2
+from pw_metric_proto import metric_service_pb2
 from pw_rpc.console_tools.console import flattened_rpc_completions
-from pw_system.device import Device
 from pw_tokenizer.detokenize import AutoUpdatingDetokenizer
+from pw_unit_test_proto import unit_test_pb2
+
+from pw_system.device import Device
 
 _LOG = logging.getLogger('tools')
 _DEVICE_LOG = logging.getLogger('rpc_device')
+_SERIAL_DEBUG = logging.getLogger('pw_console.serial_debug_logger')
+_ROOT_LOG = logging.getLogger()
 
 PW_RPC_MAX_PACKET_SIZE = 256
 SOCKET_SERVER = 'localhost'
@@ -74,7 +80,8 @@ MKFIFO_MODE = 0o666
 
 def _parse_args():
     """Parses and returns the command line arguments."""
-    parser = argparse.ArgumentParser(description=__doc__)
+    parser = argparse.ArgumentParser(prog="python -m pw_system.console",
+                                     description=__doc__)
     group = parser.add_mutually_exclusive_group(required=True)
     group.add_argument('-d', '--device', help='the serial port to use')
     parser.add_argument('-b',
@@ -110,6 +117,7 @@ def _parse_args():
                         help='Path to a pw_console yaml config file.')
     parser.add_argument('--proto-globs',
                         nargs='+',
+                        default=[],
                         help='glob pattern for .proto files')
     parser.add_argument('-v',
                         '--verbose',
@@ -124,9 +132,15 @@ def _expand_globs(globs: Iterable[str]) -> Iterator[Path]:
             yield Path(file)
 
 
-def _start_ipython_terminal(device: Device,
-                            serial_debug: bool = False,
-                            config_file_path: Optional[Path] = None) -> None:
+def _start_ipython_terminal(
+    device: Device,
+    device_log_store: LogStore,
+    root_log_store: LogStore,
+    serial_debug_log_store: LogStore,
+    log_file: str,
+    serial_debug: bool = False,
+    config_file_path: Optional[Path] = None,
+) -> None:
     """Starts an interactive IPython terminal with preset variables."""
     local_variables = dict(
         client=device.client,
@@ -154,14 +168,12 @@ def _start_ipython_terminal(device: Device,
     client_info = device.info()
     completions = flattened_rpc_completions([client_info])
 
-    log_windows = {
-        'Device Logs': [_DEVICE_LOG],
-        'Host Logs': [logging.getLogger()],
+    log_windows: Dict[str, Union[List[logging.Logger], LogStore]] = {
+        'Device Logs': device_log_store,
+        'Host Logs': root_log_store,
     }
     if serial_debug:
-        log_windows['Serial Debug'] = [
-            logging.getLogger('pw_console.serial_debug_logger')
-        ]
+        log_windows['Serial Debug'] = serial_debug_log_store
 
     interactive_console = PwConsoleEmbed(
         global_vars=local_variables,
@@ -171,16 +183,16 @@ def _start_ipython_terminal(device: Device,
         help_text=__doc__,
         config_file_path=config_file_path,
     )
-    interactive_console.hide_windows('Host Logs')
     interactive_console.add_sentence_completer(completions)
     if serial_debug:
         interactive_console.add_bottom_toolbar(BandwidthToolbar())
 
     # Setup Python logger propagation
-    interactive_console.setup_python_logging()
-
-    # Don't send device logs to the root logger.
-    _DEVICE_LOG.propagate = False
+    interactive_console.setup_python_logging(
+        # Send any unhandled log messages to the external file.
+        last_resort_filename=log_file,
+        # Don't change propagation for these loggers.
+        loggers_with_no_propagation=[_DEVICE_LOG])
 
     interactive_console.embed()
 
@@ -223,6 +235,20 @@ def console(device: str,
     if output is sys.stdout:
         output = sys.stdout.buffer
 
+    # Don't send device logs to the root logger.
+    _DEVICE_LOG.propagate = False
+    # Create pw_console LogStore handlers. These are the data source for log
+    # messages to be displayed in the UI.
+    device_log_store = LogStore()
+    root_log_store = LogStore()
+    serial_debug_log_store = LogStore()
+    # Attach the LogStores as handlers for each log window we want to show.
+    # This should be done before device initialization to capture early
+    # messages.
+    _DEVICE_LOG.addHandler(device_log_store)
+    _ROOT_LOG.addHandler(root_log_store)
+    _SERIAL_DEBUG.addHandler(serial_debug_log_store)
+
     if not logfile:
         # Create a temp logfile to prevent logs from appearing over stdout. This
         # would corrupt the prompt toolkit UI.
@@ -230,17 +256,16 @@ def console(device: str,
 
     log_level = logging.DEBUG if verbose else logging.INFO
     pw_cli.log.install(log_level, True, False, logfile)
-    _DEVICE_LOG.setLevel(log_level)
     _LOG.setLevel(log_level)
+    _DEVICE_LOG.setLevel(log_level)
+    _ROOT_LOG.setLevel(log_level)
+    _SERIAL_DEBUG.setLevel(log_level)
 
     detokenizer = None
     if token_databases:
         detokenizer = AutoUpdatingDetokenizer(*token_databases)
         detokenizer.show_errors = True
 
-    if not proto_globs:
-        proto_globs = ['**/*.proto']
-
     protos: List[Union[ModuleType, Path]] = list(_expand_globs(proto_globs))
 
     if compiled_protos is None:
@@ -250,7 +275,9 @@ def console(device: str,
     # provided, and shadowing errors due to ordering when the default global
     # search path is used.
     compiled_protos.append(log_pb2)
+    compiled_protos.append(unit_test_pb2)
     protos.extend(compiled_protos)
+    protos.append(metric_service_pb2)
 
     if not protos:
         _LOG.critical('No .proto files were found with %s',
@@ -298,7 +325,9 @@ def console(device: str,
                            timestamp_decoder=timestamp_decoder,
                            rpc_timeout_s=5)
 
-    _start_ipython_terminal(device_client, serial_debug, config_file)
+    _start_ipython_terminal(device_client, device_log_store, root_log_store,
+                            serial_debug_log_store, logfile, serial_debug,
+                            config_file)
     return 0
 
 
