@@ -118,8 +118,9 @@ class WatchApp(PluginMixin):
         self.ninja_log_pane.log_view.log_store.formatter = logging.Formatter(
             '%(message)s')
         self.ninja_log_pane.table_view = False
-        # Enable line wrapping
-        self.ninja_log_pane.toggle_wrap_lines()
+        # Disable line wrapping for improved error visibility.
+        if self.ninja_log_pane.wrap_lines:
+            self.ninja_log_pane.toggle_wrap_lines()
         # Blank right side toolbar text
         self.ninja_log_pane._pane_subtitle = ' '
         self.ninja_log_view = self.ninja_log_pane.log_view
@@ -147,7 +148,8 @@ class WatchApp(PluginMixin):
 
         self.window_manager.add_pane(self.ninja_log_pane)
 
-        self.time_waster = Twenty48Pane(self)
+        self.time_waster = Twenty48Pane(include_resize_handle=True)
+        self.time_waster.application = self
         self.time_waster.show_pane = False
         self.window_manager.add_pane(self.time_waster)
 
@@ -200,7 +202,7 @@ class WatchApp(PluginMixin):
             "Rebuild."
             self.run_build()
 
-        @key_bindings.add('c-g', filter=self.input_box_not_focused())
+        @key_bindings.add('c-t', filter=self.input_box_not_focused())
         def _pass_time(_event):
             "Rebuild."
             self.time_waster.show_pane = not self.time_waster.show_pane
@@ -306,6 +308,9 @@ class WatchApp(PluginMixin):
         self.ninja_log_view.log_store.clear_logs()
         self.ninja_log_view._restart_filtering()  # pylint: disable=protected-access
         self.ninja_log_view.view_mode_changed()
+        # Re-enable follow if needed
+        if not self.ninja_log_view.follow:
+            self.ninja_log_view.toggle_follow()
 
     def run_build(self):
         """Manually trigger a rebuild."""
